#include <cv.h>
#include <highgui.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>

#define FFTN 1024
#define SCALEX 640
#define SCALEY 320
#define OFTX 40
#define OFTY 20
#define STEPX 4

void DrawScale(CvArr* img);
void FftCal(float *iW, float *oR, float *oI, float *oP);
void TxtLoad(char *fn, float *iW);
void TxtSave(char *fn, float *iW);
void WinFuncCal(int wintype);
float WinFunc[FFTN];
int WinType = 0;
float MAG = 2.0;
float DIV = 10000.0;

int main(int argc, char *argv[]) {

  float InWave[FFTN];
  float RealSpec[FFTN];
  float ImagSpec[FFTN];
  float PowerSpec[FFTN];
  char input_file[256];
  IplImage *img;
  int val, valp1;
  int i;

  img = cvCreateImage (cvSize (720, 360), 8, 3);

  cvSet (img, cvScalarAll (255), 0);

  if (argc < 2) {
      fprintf(stderr, "1st arg must be input file name.\n");
      return -1;
  }

  strcpy(input_file, argv[1]);
  TxtLoad(input_file, InWave);

	WinFuncCal(WinType);
        FftCal(InWave, RealSpec, ImagSpec, PowerSpec);

    cvRectangle (img,
               cvPoint (OFTX, OFTY),
               cvPoint (OFTX+SCALEX, OFTY+SCALEY),
               CV_RGB(255,255,255), CV_FILLED, 8, 0);

  CvScalar color = CV_RGB(255, 0, 0);

    while(1) {
        DrawScale(img);

        for(i = 0; i < 500; i++) {
          if(STEPX*i < SCALEX) {
            val = (int)(10*log10(PowerSpec[i]/DIV)*MAG);
            valp1 = (int)(10*log10(PowerSpec[i+1]/DIV)*MAG);
            cvLine( img, cvPoint(OFTX+STEPX*i, OFTY+SCALEY*3/4-val),
                         cvPoint(OFTX+STEPX*i+STEPX, OFTY+SCALEY*3/4-valp1),
                         color, 1, 8, 0);
          }
        }

        cvNamedWindow ("Spectrum", CV_WINDOW_AUTOSIZE);
        cvShowImage ("Spectrum", img);
        if(cvWaitKey(1)>=0) break;
    }

        TxtSave("fft.txt", PowerSpec);

  return 0;
}

void FftCal(float *iW, float *oR, float *oI, float *oP)
{
        short N = FFTN; 
        short P; 
        short N_2 = N/2;
        short i,j,k,kp,m,h;
        float w1, w2;
        float s1,s2;
        float t1,t2;
        float fReal[FFTN], fImag[FFTN];
        float tri[FFTN];

        i = N; P = 0;
        while (i != 1) {
                i = i / 2;
                P++;
        }

        for (i = 0; i < N; i++) {
                fReal[i] = iW[i] * WinFunc[i];
		fImag[i] = 0;
        }

       for ( i = 0; i < N_2; i++ ) {
                tri[i] = cos( 2 * i * M_PI / N );
                tri[i + N_2] = (-1.0) * sin( 2 * i * M_PI / N );
        }

        j = 0;
        for ( i = 0; i <= N-2; i++ ) {
                if (i < j) {
                        t1 = fReal[j]; fReal[j] = fReal[i]; fReal[i] = t1;
                }
                k = N_2;
                while (k <= j) {
                        j = j - k; k = k/2;
                }
                j = j + k;
        }

        for ( i = 1; i <= P; i++ ) {
                m = pow(2, i);
                h = m/2;
                for ( j = 0; j < h; j++ ) {
                        w1 = tri[j*(N/m)];
                        w2 = tri[j*(N/m) + N_2];
                        for( k = j; k < N; k+=m ) {
                                kp = k + h;
                                s1 = fReal[kp] * w1 - fImag[kp] * w2;
                                s2 = fReal[kp] * w2 + fImag[kp] * w1;
                                t1 = fReal[k] + s1; fReal[kp] = fReal[k] - s1; fReal[k] = t1;
                                t2 = fImag[k] + s2; fImag[kp] = fImag[k] - s2; fImag[k] = t2;
                        }
                }
        }
        for ( i = 0; i < N; i++ ) {
                oR[i] = fReal[i];
                oI[i] = fImag[i];
                oP[i] = fReal[i] * fReal[i] + fImag[i] * fImag[i];
        }
}

void TxtLoad(char *fn, float *iW)
{
  int count = 0;
  char charline[40];
  FILE *fp;

  fp = fopen( fn, "r" );
  if( fp == NULL ){
    printf( "%s cannot be opened.\n", fn );
    exit(1);
  } 

  fseek( fp, 0L, SEEK_SET );
  while( fgets(charline, 40, fp) != NULL ) {
    iW[count] = atof(charline);
//    printf( "%d  %f\n", count, iW[count] );
    count++;

    if(count == FFTN) {
      break;
    }
  } 
  fclose( fp );
}

void TxtSave(char *fn, float *iW)
{
  int i;
  char charline[40];
  FILE *fp;

  fp = fopen( fn, "w" );
  if( fp == NULL ){
    printf( "%s cannot be opened.\n", fn );
    exit(1);
  } 

  fseek( fp, 0L, SEEK_SET );
  for( i = 0; i < FFTN; i++) {
    sprintf(charline, "%f\n", iW[i]);
    fputs(charline, fp);
//    printf( "%d\n", iW[i] );
  } 
  fclose( fp );
}

void WinFuncCal(int wintype)
{
        int N = FFTN; 
        int i;
        double Pi = 3.1415926;

        if( wintype == 1 ) { // Hamming
                for( i = 0; i < N; i++ ) {
                        WinFunc[i] = 0.54 - 0.46 * cos(Pi * 2 * i / (N-1)); 
                }
        } else if( wintype == 2 ) { // Hanning
                for( i = 0; i < N; i++ ) {
                        WinFunc[i] = 0.5 * ( 1.0 - cos(Pi * 2 * i / (N-1)) ); 
                }
        } else if( wintype == 3 ) { // Blackman
                for( i = 0; i < N; i++ ) {
                        WinFunc[i] = 0.42 - 0.5 * cos(Pi * 2 * i / (N-1)) 
                                        + 0.08 * cos(2.0 * Pi * 2 * i / (N-1)); 
                }
        } else if( wintype == 4 ) { // Blackman-Harris
                double a0 = 0.35875;
                double a1 = 0.48829;
                double a2 = 0.14128;
                double a3 = 0.01168;
                for( i = 0; i < N; i++ ) {
                        WinFunc[i] = a0 - a1 * cos(Pi * 2 * (i+0.5) / N) 
                                        + a2 * cos(Pi * 2 * 2.0 * (i+0.5) / N) 
                                        - a3 * cos(Pi * 2 * 3.0 * (i+0.5) / N);
                }
        } else if( wintype == 5 ) { // Bartlett
                for( i = 0; i <= (N-1)/2; i++ ) {
                        WinFunc[i] = 2.0 * i / (N-1);
                }
                for( i = (N-1)/2; i < N; i++ ) {
                        WinFunc[i] = 2.0 - 2.0 * i / (N-1);
                }
        } else { // No Window (Rectangle)
                for( i = 0; i < N; i++ ) {
                        WinFunc[i] = 1.0; 
                }
        }
}

void DrawScale(CvArr* img) {
  float fullperiod = 7500;
  int i;

  CvFont font;
  int font_face[] = {
    CV_FONT_HERSHEY_SIMPLEX,
    CV_FONT_HERSHEY_PLAIN,
    CV_FONT_HERSHEY_DUPLEX,
    CV_FONT_HERSHEY_COMPLEX,
    CV_FONT_HERSHEY_TRIPLEX,
    CV_FONT_HERSHEY_COMPLEX_SMALL,
    CV_FONT_HERSHEY_SCRIPT_SIMPLEX,
    CV_FONT_HERSHEY_SCRIPT_COMPLEX
  };
  char tmpchar[16];

  cvRectangle (img,
               cvPoint (OFTX, OFTY),
               cvPoint (OFTX+SCALEX, OFTY+SCALEY),
               cvScalarAll (0), 1, 8, 0);

  cvInitFont(&font, font_face[0], 0.5, 0.5, 0, 1, 8);

  sprintf( tmpchar, "0Hz");
  cvPutText(img, tmpchar, cvPoint(OFTX + 3, OFTY + SCALEY - 10) , &font, CV_RGB(0, 0, 0));
  sprintf( tmpchar, "%dHz", (int)(fullperiod*1/3) );
  cvPutText(img, tmpchar, cvPoint(OFTX + SCALEX/3 - 15, OFTY + SCALEY - 10) , &font, CV_RGB(0, 0, 0));
  sprintf( tmpchar, "%dHz", (int)(fullperiod*2/3) );
  cvPutText(img, tmpchar, cvPoint(OFTX + SCALEX*2/3 - 15, OFTY + SCALEY - 10) , &font, CV_RGB(0, 0, 0));
  sprintf( tmpchar, "%dHz", (int)(fullperiod*3/3) );
  cvPutText(img, tmpchar, cvPoint(OFTX + SCALEX - 35, OFTY + SCALEY - 10) , &font, CV_RGB(0, 0, 0));

  sprintf( tmpchar, "%d", -20);
  cvPutText(img, tmpchar, cvPoint(3, OFTY + SCALEY*3/4+20*MAG+5 ) , &font, CV_RGB(0, 0, 0));
  sprintf( tmpchar, "%d", 0);
  cvPutText(img, tmpchar, cvPoint(3, OFTY + SCALEY*3/4-0*MAG+5 ) , &font, CV_RGB(0, 0, 0));
  sprintf( tmpchar, "%d", 20);
  cvPutText(img, tmpchar, cvPoint(3, OFTY + SCALEY*3/4-20*MAG+5 ) , &font, CV_RGB(0, 0, 0));
  sprintf( tmpchar, "%d", 40);
  cvPutText(img, tmpchar, cvPoint(3, OFTY + SCALEY*3/4-40*MAG+5 ) , &font, CV_RGB(0, 0, 0));
  sprintf( tmpchar, "%d", 60);
  cvPutText(img, tmpchar, cvPoint(3, OFTY + SCALEY*3/4-60*MAG+5 ) , &font, CV_RGB(0, 0, 0));
  sprintf( tmpchar, "%d", 80);
  cvPutText(img, tmpchar, cvPoint(3, OFTY + SCALEY*3/4-80*MAG+5 ) , &font, CV_RGB(0, 0, 0));
  sprintf( tmpchar, "100");
  cvPutText(img, tmpchar, cvPoint(3, OFTY + SCALEY*3/4-100*MAG+5 ) , &font, CV_RGB(0, 0, 0));
  sprintf( tmpchar, "[dB]");
  cvPutText(img, tmpchar, cvPoint(3, OFTY + SCALEY*3/4-120*MAG+5 ) , &font, CV_RGB(0, 0, 0));

  for(i = 1; i <= 14; i++) {
    cvLine( img, cvPoint(OFTX + SCALEX*i/15, OFTY + SCALEY),
                   cvPoint(OFTX + SCALEX*i/15, OFTY),
                   CV_RGB(230, 230, 230), 1, 8, 0);
  }

  for(i = 1; i <= 7; i++) {
    cvLine( img, cvPoint(OFTX, OFTY + SCALEY*i/8),
                   cvPoint(OFTX + SCALEX, OFTY + SCALEY*i/8),
                   CV_RGB(230, 230, 230), 1, 8, 0);
  }

  cvLine( img, cvPoint(OFTX + SCALEX/3, OFTY + SCALEY),
                   cvPoint(OFTX + SCALEX/3, OFTY),
                   CV_RGB(200, 200, 200), 1, 8, 0);
  cvLine( img, cvPoint(OFTX + SCALEX*2/3, OFTY + SCALEY),
                   cvPoint(OFTX + SCALEX*2/3, OFTY),
                   CV_RGB(200, 200, 200), 1, 8, 0);
  cvLine( img, cvPoint(OFTX, OFTY + SCALEY*3/4),
                   cvPoint(OFTX + SCALEX, OFTY + SCALEY*3/4),
                   CV_RGB(200, 200, 200), 1, 8, 0);
}

